----------------------------------------------------------------------------------
-- Company: 
-- Engineer: Liam Gow
-- 
-- Create Date:    10:30:52 11/19/2014 
-- Design Name: 
-- Module Name:    Buzzer - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
-- Buzzer is intended to be used in conjunction with a piezoelectric buzzer
-- Buzzer triggers beeps with frequency roughly inversely proportional to the time remaining of a static time limit (changable below)
-- Buzzer will also beep three times in rapid succession if the win condition of the game is met to signal the end of the game
-- Buzzer will hold a tone for 5 seconds if the time limit runs out without the win condition having been met
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity Buzzer is
    Port ( Timer : in STD_LOGIC_VECTOR (7 downto 0);
			TenthCLK : in STD_LOGIC;
				Win : in STD_LOGIC;
				limit : in STD_LOGIC_VECTOR (7 downto 0);
           PinOut : out  STD_LOGIC := '0');
end Buzzer;

architecture Behavioral of Buzzer is

	signal countdown : integer;
	signal timeLimit : integer;
	
	signal trigger : std_logic_vector (1 downto 0);

begin

	countdown <= to_integer(unsigned(timer));
	timeLimit <= to_integer(unsigned(limit));

-- This process  assigns discrete values to trigger based on the fraction of the time limit that has elapsed
trigger_type : process (countdown)

begin

	if countdown > (timeLimit/2) then -- time/2
		trigger <= "00";
	elsif countdown > (timeLimit/4) then -- time/4
		trigger <= "01";
	elsif countdown > (timeLimit/10) then --time/10
		trigger <= "10";
	elsif countdown > 0 then
		trigger <= "11";
	else trigger <= "00";
	end if;
		
end process;
	
-- This process beeps at a frequency based on the trigger signal given
-- The current periods are 0.5 seconds, 1 second, and 2 seconds
beep : process (TenthCLK, trigger)
	
	variable two_count : integer := 0;
	variable one_count : integer := 0;
	variable half_count : integer := 0;
	variable lose_count : integer := 0;
	variable win_count : integer := 0;
	variable beep_num : integer := 0;
	
begin
	if rising_edge(tenthCLK) then
		-- Victory Condition
		if win = '1' and beep_num < 3 then -- Number of victory beeps
			if win_count = 10 then -- Beeps once per second
				win_count := 0;
				pinout <= '1';
				beep_num := beep_num + 1;
			else 
				win_count := win_count + 1;
				pinout <= '0';
			end if;
			
		-- Normal Operating Condition
		elsif countdown > 0 then
			case trigger is
				when "00" => -- No beep
					pinout <= '0';
					
				when "01" => -- Beeps once per two seconds
					if two_count = 20 then
						two_count := 0;
						pinout <= '1';
					else 
						two_count := two_count + 1;
						pinout <= '0';
					end if;
					
				when "10" => -- Beeps once per second
					if one_count = 10 then
						one_count := 0;
						pinout <= '1';
					else 
						one_count := one_count + 1;
						pinout <= '0';
					end if;
					
				when "11" => -- Beeps once per half second
					if half_count = 5 then
						half_count := 0;
						pinout <= '1';
					else 
						half_count := half_count + 1;
						pinout <= '0';
					end if;
					
				when others => pinout <= '0';
			end case;
			
		-- Lose Condition
		else 
			if lose_count = 50 then -- Holds tone for 5 seconds
				pinout <= '0';
				
			else 
				lose_count := lose_count + 1;
				pinout <= '1';
			end if;
		end if;
	end if;
	
end process;
	
end Behavioral;

