----------------------------------------------------------------------------------
-- Company: 
-- Engineer: Liam Gow
-- 
-- Create Date:    11:35:12 11/17/2014 
-- Design Name: 
-- Module Name:    LEDDriver - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
-- LED Driver takes in the player's X and Y positions, and assigns high and low signals to only the appropriate Power(+) and Ground(-) lines in the LED grid.

-- This code is capable of dealing with up to 16 x 16 mazes, though replacing the 15s with other numbers should make it compatible.
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity LEDDriver is
    Port ( PlayerX : in  STD_LOGIC_VECTOR (7 downto 0);
           PlayerY : in  STD_LOGIC_VECTOR (7 downto 0);
           Power : out  STD_LOGIC_VECTOR (15 downto 0);
           Ground : out  STD_LOGIC_VECTOR (15 downto 0));
end LEDDriver;

architecture Behavioral of LEDDriver is

begin

light_led : process (playerX, playerY)


begin

	-- The input X and Y positions are divided by two due to the array also containing wall spaces
	-- Due to integer division truncation, this will always find the correct value
	
	-- Loops through all X values, checking for a match with the player's X value
	-- If a match is found, the high signal is applied
	-- All other X lines are driven low
	for I in 0 to 15 loop
		if I = ((to_integer(unsigned(playerX)))/2) then 
			Power(I) <= '1';
		else Power(I) <= '0';
		end if;
	end loop;
	
	-- Loops through all Y values, checking for a match with the player's Y value
	-- If a match is found, the low signal is applied
	-- All other X lines are driven high
	for J in 0 to 15 loop
		if J = ((to_integer(unsigned(playerY)))/2) then
			Ground(J) <= '0';
		else Ground(J) <= '1';
		end if;
	end loop;
	
end process;

end Behavioral;

