----------------------------------------------------------------------------------
-- Company: 
-- Engineer: Liam Gow
-- 
-- Create Date:    11:16:37 11/17/2014 
-- Design Name: 
-- Module Name:    Timing - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
-- Timing controls clocks of different speeds used in other modules
-- The currently implemented clocks have periods 0.1 seconds, 1 second, and 0.4 seconds

-- These clocks assume a base clock frequency of 100Mhz.  Any other clock will need to be accounted for in the max count values
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity Timing is
    Port ( CLK : in  STD_LOGIC;
           TenthCLK : out  STD_LOGIC;
           SecondCLK : out  STD_LOGIC;
			  Iterate : out STD_LOGIC);
end Timing;

architecture Behavioral of Timing is

begin

timing : process (clk)

-- variable declaration --
	variable tenth : integer := 0;
	variable slow_count : integer := 0;
	variable seconds : integer := 0;
	variable SlowCLK : std_logic := '0';
	variable tempSecondCLK : std_logic := '0';
	variable tempIterate : std_logic := '0';
	variable nextIterate : std_logic := '0';
	
begin

	if(rising_edge(CLK)) then
		-- slow_count oscillates every 50 ms
		slow_count := slow_count + 1;
		if slow_count = 5000000 then
			slow_count := 0;
			slowCLK := not slowCLK;
			tenthCLK <= slowCLK;
			tenth := tenth + 1;
			
		-- secondCLK oscillates every 0.5 seconds
			if tenth = 10 then
				secondCLK <= not tempSecondCLK;
				tempSecondCLK := not tempSecondCLK;
				tenth := 0;
			end if;
		
	
			-- updates every 0.2 seconds : can be changed to suit button sensitivity preferences

			if tenth = 0 or tenth = 2 or tenth = 4 or tenth = 6 or tenth = 8 then
				tempIterate := nextIterate;
				nextIterate := not nextIterate;
			end if;
		end if;
	end if;
	
	-- Assigns iterate signal to output
	Iterate <= tempIterate;
	

	
end process;

end Behavioral;

